unit UBC;

interface

uses
    Forms, Classes, Controls, StdCtrls, ExtCtrls, Menus, Types, Graphics,
    Word97, OleServer, Clipbrd, Dialogs, SysUtils, Variants, Registry, Windows,
    ShellAPI, BarCodes, IniFiles;

const
     NUMBERS: string = '1234567890';
     AMOUNT = 10;
     BC_TOP = 5;
     BC_LEFT = 5;
     BC_HEIGHT = 50;

     REG_KEY = '\Software\Yugene Egorov\BarCode Editor';
     REG_TEMPLATEFILE = 'TemplateFileName';
     REG_SAVETO = 'SaveToTemplate';
     REG_SHOWWORD = 'ShowWordWhenFinished';
     REG_FILES = 'NumberOfFiles';
     REG_TYPE = 'CodeType';
     REG_WIDTH = 'Thickness';
     REG_SAVEOPT = 'SaveOptionsTo';

     DEF_TEMPLATEFILE = 'c:\test.doc';
     DEF_SAVETO = 'c:\reader-';
     DEF_SHOWWORD = FALSE;
     DEF_FILES = 10;
     DEF_TYPE = 2;
     DEF_WIDTH = 0;
     DEF_SAVEOPT = 1;

     INI_SECTION = 'Options';

type
  TMainForm = class(TForm)
    Label2: TLabel;
    StartBCEdit: TEdit;
    ExitButton: TButton;
    WordApplication1: TWordApplication;
    WordDocument1: TWordDocument;
    WorkButton: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Program1: TMenuItem;
    About1: TMenuItem;
    ViewButton: TButton;
    N1: TMenuItem;
    N2: TMenuItem;
    Hlp: TMenuItem;
    procedure WorkButtonClick(Sender: TObject);
    procedure ExitButtonClick(Sender: TObject);
    procedure StartBCEditKeyPress(Sender: TObject; var Key: Char);
    procedure ViewButtonClick(Sender: TObject);
    procedure N1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure HlpClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  TemplateFile: string;
  SaveTo: string;
  ShowWord: Boolean;
  NFiles: Integer;
  BC: TBarCode;
  bcType: Integer;
  bcTypes: array[0..4] of TBarCodeType;
  Thickness: Integer;
  SaveOptions: Integer;

implementation

uses UBC2, UOptions, UAbout, UHelp;

{$R *.dfm}

function NextBC(BC: string): string;
var
   S: string;
procedure NextChar(P: Integer);
var
   I: Integer;
begin
     if (Pos(S[P], NUMBERS) <> 0) then
     begin
          I := StrToInt(S[P]);
          S[P] := IntToStr((I + 1) mod 10)[1];
          if (I = 9) then NextChar(P - 1);             
     end;
end;
begin
     S := BC;
     NextChar(Length(S));
     NextBC := S;
end;

function BCWidth(Str, BCStr: string): Integer;
var
   Width: Integer;
   Thick: Integer;
   I: Integer;
begin
     Width := 0;
     Thick := 2;
     case (Thickness) of
     0: Thick := 2;
     1: Thick := 3;
     end;
     for I := 1 to Length(BCStr) do
     case (BCStr[I]) of
     '0': Width := Width + 1;
     '1': Width := Width + 1;
     '2': Width := Width + Thick;
     '3': Width := Width + Thick;
     end;
     BCWidth := Width + Length(Str) - 1;
end;

procedure CreateBarCode(Text: string);
var
   Offset: Integer;
   S: string;
   J: Integer;
   W: Integer;
begin
     Offset := BC_LEFT;
     S := BC.Produce(Text, bcTypes[bcType]);
     W := BCWidth(Text, S);
     with CodeForm.Image1.Canvas do
     begin
          Brush.Color := clWhite;
          Pen.Color := clWhite;
          Rectangle(1, 1, BC_LEFT + W + BC_LEFT, BC_TOP + BC_HEIGHT + BC_TOP + TextHeight(Text) + BC_TOP);
          Pen.Color := clBlack;
     end;

     with CodeForm do
     begin
          with Image1 do
          begin
               Picture.Bitmap.Width := BC_LEFT + W + BC_LEFT;
               Picture.Bitmap.Height := BC_TOP + BC_HEIGHT + BC_TOP + CodeForm.Image1.Canvas.TextHeight(Text) + BC_TOP;
               Canvas.TextOut((W div 2) - (Canvas.TextWidth(Text) div 2) + BC_LEFT, BC_TOP + BC_HEIGHT + BC_TOP, Text);               
          end;
          ClientWidth := CodeForm.Image1.Picture.Bitmap.Width;
          ClientHeight := CodeForm.Image1.Picture.Bitmap.Height + CodeForm.StatusBar1.Height;
          Caption := Text;
     end;

     for J := 1 to Length(S) do
     begin
          CodeForm.Image1.Canvas.MoveTo(Offset, BC_TOP);
          case (S[J]) of
          '0': Offset := Offset + 1;
          '1':
          begin
               CodeForm.Image1.Canvas.LineTo(Offset, BC_TOP + BC_HEIGHT);
               Offset := Offset + 1;
          end;
          '2':
          begin
               case (Thickness) of
               0: Offset := Offset + 2;
               1: Offset := Offset + 3;
               end;
          end;
          '3':
          begin
               case (Thickness) of
               0:
               begin
                    with CodeForm.Image1.Canvas do
                    begin
                         LineTo(Offset, BC_TOP + BC_HEIGHT);
                         Offset := Offset + 1;
                         MoveTo(Offset, BC_TOP);
                         LineTo(Offset, BC_TOP + BC_HEIGHT);
                         Offset := Offset + 1;
                    end;
               end;
               1:
               begin
                    with CodeForm.Image1.Canvas do
                    begin
                         LineTo(Offset, BC_TOP + BC_HEIGHT);
                         Offset := Offset + 1;
                         MoveTo(Offset, BC_TOP);
                         LineTo(Offset, BC_TOP + BC_HEIGHT);
                         Offset := Offset + 1;
                         MoveTo(Offset, BC_TOP);
                         LineTo(Offset, BC_TOP + BC_HEIGHT);
                         Offset := Offset + 1;
                    end;
               end;
               end;
          end;
          end;
     end;
end;

procedure LoadConfig;
var
   Reg: TRegistry;
   Ini: TIniFile;
begin
     Reg := TRegistry.Create;
     Reg.RootKey := HKEY_CURRENT_USER;
     if (Reg.OpenKey(REG_KEY, FALSE)) then
     begin
          if (Reg.ValueExists(REG_SAVEOPT)) then SaveOptions := Reg.ReadInteger(REG_SAVEOPT)
          else SaveOptions := DEF_SAVEOPT;
     end
     else SaveOptions := DEF_SAVEOPT;
     case (SaveOptions) of
     0:
     begin
          if (Reg.OpenKey(REG_KEY, FALSE)) then
          begin
               if (Reg.ValueExists(REG_TEMPLATEFILE)) then TemplateFile := Reg.ReadString(REG_TEMPLATEFILE)
               else TemplateFile := DEF_TEMPLATEFILE;
               if (Reg.ValueExists(REG_SAVETO)) then SaveTo := Reg.ReadString(REG_SAVETO)
               else SaveTo := DEF_SAVETO;
               if (Reg.ValueExists(REG_SHOWWORD)) then ShowWord := Reg.ReadBool(REG_SHOWWORD)
               else ShowWord := DEF_SHOWWORD;
               if (Reg.ValueExists(REG_FILES)) then NFiles := Reg.ReadInteger(REG_FILES)
               else NFiles := DEF_FILES;
               if (Reg.ValueExists(REG_TYPE)) then bcType := Reg.ReadInteger(REG_TYPE)
               else NFiles := DEF_TYPE;
               if (Reg.ValueExists(REG_WIDTH)) then Thickness := Reg.ReadInteger(REG_WIDTH)
               else Thickness := DEF_WIDTH;
               Reg.CloseKey;
          end
          else
          begin
               TemplateFile := DEF_TEMPLATEFILE;
               SaveTo := DEF_SAVETO;
               ShowWord := DEF_SHOWWORD;
               NFiles := DEF_FILES;
               bcType := DEF_TYPE;
               Thickness := DEF_WIDTH;
          end;
     end;
     1:
     begin
          Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
          TemplateFile := Ini.ReadString(INI_SECTION, REG_TEMPLATEFILE, DEF_TEMPLATEFILE);
          SaveTo := Ini.ReadString(INI_SECTION, REG_SAVETO, DEF_SAVETO);
          ShowWord := Ini.ReadBool(INI_SECTION, REG_SHOWWORD, DEF_SHOWWORD);
          NFiles := Ini.ReadInteger(INI_SECTION, REG_FILES, DEF_FILES);
          bcType := Ini.ReadInteger(INI_SECTION, REG_TYPE, DEF_TYPE);
          Thickness := Ini.ReadInteger(INI_SECTION, REG_WIDTH, DEF_WIDTH);
          Ini.Free;
     end;
     end;
     Reg.Free;
end;

procedure SaveConfig;
var
   Reg: TRegistry;
   Ini: TIniFile;
begin
     Reg := TRegistry.Create;
     Reg.RootKey := HKEY_CURRENT_USER;
     if (Reg.OpenKey(REG_KEY, TRUE)) then Reg.WriteInteger(REG_SAVEOPT, SaveOptions);
     case (SaveOptions) of
     0:
     begin
          Reg.WriteString(REG_TEMPLATEFILE, TemplateFile);
          Reg.WriteString(REG_SAVETO, SaveTo);
          Reg.WriteBool(REG_SHOWWORD, ShowWord);
          Reg.WriteInteger(REG_FILES, NFiles);
          Reg.WriteInteger(REG_TYPE, bcType);
          Reg.WriteInteger(REG_WIDTH, Thickness);
     end;
     1:
     begin
          Ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
          Ini.WriteString(INI_SECTION, REG_TEMPLATEFILE, TemplateFile);
          Ini.WriteString(INI_SECTION, REG_SAVETO, SaveTo);
          Ini.WriteBool(INI_SECTION, REG_SHOWWORD, ShowWord);
          Ini.WriteInteger(INI_SECTION, REG_FILES, NFiles);
          Ini.WriteInteger(INI_SECTION, REG_TYPE, bcType);
          Ini.WriteInteger(INI_SECTION, REG_WIDTH, Thickness);
          Ini.Free;
     end;
     end;
     Reg.CloseKey;     
     Reg.Free;
end;

procedure TMainForm.ExitButtonClick(Sender: TObject);
begin
     Application.Terminate;
end;

procedure DisableButtons;
begin
     MainForm.WorkButton.Enabled := FALSE;
     MainForm.ViewButton.Enabled := FALSE;
     MainForm.ExitButton.Enabled := FALSE;
end;

procedure EnableButtons;
begin
     MainForm.WorkButton.Enabled := TRUE;
     MainForm.ViewButton.Enabled := TRUE;
     MainForm.ExitButton.Enabled := TRUE;
end;

procedure TMainForm.WorkButtonClick(Sender: TObject);
var
   C: TClipboard;
   OV: OleVariant;
   I, K: Integer;
   StartBC, BC: string;
begin
     if (FileExists(TemplateFile)) then
     try
        try
           try
              Wordapplication1.Connect;
           except
                 MessageDlg('Word may not be installed', mtError, [mbOk], 0);
                 Abort;
           end;
           Wordapplication1.Visible := FALSE;
           WordApplication1.Caption := ' ';
           WordApplication1.Options.CheckSpellingAsYouType := False;
           WordApplication1.Options.CheckGrammarAsYouType := False;
           BC := StartBCEdit.Text;
           DisableButtons;
           C := TClipboard.Create;

           for K := 1 to NFiles do
           begin
                StartBC := BC;
                OV := TemplateFile;
                WordApplication1.Documents.Open(OV, EmptyParam, EmptyParam, EmptyParam, EmptyParam, EmptyParam, EmptyParam, EmptyParam, EmptyParam, EmptyParam);
                WordDocument1.ConnectTo(WordApplication1.ActiveDocument);

                for I := 1 to AMOUNT do
                begin
                     CreateBarCode(BC);
                     C.Assign(CodeForm.Image1.Picture.Bitmap);
                     WordApplication1.Run('Img' + IntToStr(I));
                     if (I <> AMOUNT) then BC := NextBC(BC);
                end;
                OV := SaveTo + StartBC + '-' + BC + '.doc';
                WordDocument1.SaveAs(OV);
                if (not ShowWord) then
                begin
                     OV := TRUE;
                     WordDocument1.Close(OV);
                end;
                BC := NextBC(BC);
           end;

           C.Free;
           EnableButtons;
        except
              on E: Exception do
              begin
                   ShowMessage(E.Message);
                   WordApplication1.Disconnect;
              end;
        end;
     finally
            WordDocument1.Disconnect;
            WordApplication1.Disconnect;
            Application.MessageBox(' ', '', 0);
            if (ShowWord) then WordApplication1.Visible := TRUE;            
     end
     else Application.MessageBox('  - !', '', MB_OK or MB_ICONEXCLAMATION);
end;

procedure TMainForm.StartBCEditKeyPress(Sender: TObject; var Key: Char);
begin
     if ((Pos(UpCase(Key), BC.Alphabet(bcTypes[bcType])) = 0) and (Key <> #8)) then Key := #0;
end;

procedure TMainForm.ViewButtonClick(Sender: TObject);
begin
     CreateBarCode(StartBCEdit.Text);
     CodeForm.ShowModal;
end;

procedure TMainForm.N1Click(Sender: TObject);
begin
     if (OptionsForm.ShowModal = mrOk) then
     begin
          TemplateFile := OptionsForm.TemplateFName.Text;
          SaveTo := OptionsForm.SaveFName.Text;
          ShowWord := OptionsForm.ShowWordChk.Checked;
          NFiles := OptionsForm.NFilesSpin.Value;
          bcType := OptionsForm.TypeComboBox.ItemIndex;
          Thickness := OptionsForm.WidthComboBox.ItemIndex;
          SaveOptions := OptionsForm.SaveRadio.ItemIndex; 
     end;
     SaveConfig;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
     BC := TBarCode.Create;
     bcTypes[0] := bcCodabar;
     bcTypes[1] := bcCode25;
     bcTypes[2] := bcCode39;
     bcTypes[3] := bcCode128;
     bcTypes[4] := bcEAN13;
     LoadConfig;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     if (not ShowWord) then WordApplication1.Quit;
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
     AboutBox.ShowModal;
end;

procedure TMainForm.HlpClick(Sender: TObject);
begin
     HelpForm.ShowModal;
end;

end.
